@@ -17,6 +17,7 @@ except NameError:
     string_types = str         # Python 3
 
 
+Blank = namedtuple('Blank', [])
 Special = namedtuple('Special', [])
 NamedQuery = namedtuple('NamedQuery', [])
 Database = namedtuple('Database', [])
@@ -406,7 +407,7 @@ def suggest_based_on_last_token(token, stmt):
     if token_v == 'set':
         return (Column(table_refs=stmt.get_tables(),
                        local_tables=stmt.local_tables),)
-    if token_v in ('select', 'where', 'having', 'by', 'distinct'):
+    if token_v in ('select', 'where', 'having', 'order by', 'distinct'):
         # Check for a table alias or schema qualification
         parent = stmt.identifier.get_parent_name() \
             if (stmt.identifier and stmt.identifier.get_parent_name()) else []
@@ -547,6 +548,8 @@ def suggest_based_on_last_token(token, stmt):
         return tuple(suggestions)
     if token_v in {'alter', 'create', 'drop'}:
         return (Keyword(token_v.upper()),)
+    if token_v in {'limit'}:
+        return (Blank(),)
     if token.is_keyword:
         # token is a keyword we haven't implemented any special handling for
         # go backwards in the query until we find one we do recognize
