@@ -4,7 +4,7 @@ from cyanodbc import Connection
 from typing import List, Optional, Callable
 from logging import getLogger
 from asyncio import get_event_loop
-from threading import Thread
+from threading import Thread, Lock
 from prompt_toolkit.layout.containers import HSplit, Window, ScrollOffsets, ConditionalContainer, Container
 from prompt_toolkit.formatted_text.base import StyleAndTextTuples
 from prompt_toolkit.formatted_text import fragment_list_width
@@ -40,12 +40,14 @@ class myDBObject:
         self.conn = conn
         self.children = children
         self.parent = parent
+        self.next_object = next_object
+        # Held while modifying children, parent, next_object
+        # As some of thes operatins (expand) happen asynchronously
+        self._lock = Lock()
         self.name = name
         self.otype = otype
-        self.next_object = next_object
         self.level = level
         self.selected: bool = False
-        self.logger = getLogger(__name__)
 
     def _expand_internal(self) -> None:
         """
@@ -67,6 +69,7 @@ class myDBObject:
             """ Callback, scheduled after threaded I/O
                 completes """
             self.my_app.show_expanding_object = False
+            self.my_app.obj_list_changed = True
             self.my_app.application.invalidate()
 
         def _run():
@@ -91,20 +94,24 @@ class myDBObject:
             obj = self.children[len(self.children) - 1].next_object
             while obj.level > self.level:
                 obj = obj.next_object
-            self.next_object = obj
-            self.children = None
+            with self._lock:
+                self.next_object = obj
+                self.children = None
         elif self.parent is not None:
             self.my_app.selected_object = self.parent
             self.parent.collapse()
 
+        self.my_app.obj_list_changed = True
+
     def add_children(self, list_obj: List["myDBObject"]) -> None:
         lst = list(filter(lambda x: x.name != "", list_obj))
         if len(lst):
-            self.children = lst
-            for i in range(len(self.children) - 1):
-                self.children[i].next_object = self.children[i + 1]
-            self.children[len(self.children) - 1].next_object = self.next_object
-            self.next_object = self.children[0]
+            with self._lock:
+                self.children = lst
+                for i in range(len(self.children) - 1):
+                    self.children[i].next_object = self.children[i + 1]
+                self.children[len(self.children) - 1].next_object = self.next_object
+                self.next_object = self.children[0]
 
 class myDBColumn(myDBObject):
     def _expand_internal(self) -> None:
@@ -520,35 +527,24 @@ def sql_sidebar(my_app: "sqlApp") -> Window:
         tokens.append(("class:sidebar.status" + sel + act, " " * 10))
         return tokens
 
-    def _buffer_pos_changed(buff):
-        """ When the cursor changes in the sidebar buffer, make sure the appropriate
-            database object is market as selected
-        """
-        # Only when this buffer has the focus.
-        try:
-            line_no = buff.document.cursor_position_row
-
-            if line_no < 0:  # When the cursor is above the inserted region.
-                raise IndexError
-
-            idx = 0
-            obj = my_app.obj_list[0]
-            while idx < line_no:
-                if not obj.next_object:
-                    raise IndexError
-                idx += 1
-                obj = obj.next_object
-
-            my_app.selected_object = obj
-
-        except IndexError:
-            pass
-
     search_buffer = Buffer(name = "sidebarsearchbuffer")
     search_field = SearchToolbar(
         search_buffer = search_buffer,
         ignore_case = True
     )
+    def _buffer_pos_changed(buff):
+        """ This callback gets executed after cursor position changes.  Most
+            of the time we register a key-press (up / down), we change the
+            selected object and as a result of that the cursor changes.  By that
+            time we don't need to updat the selected object (cursor changed as
+            a result of the selected object being updated).  The one exception
+            is when searching the sidebar buffer.  When this happens the cursor
+            moves ahead of the selected object.  When that happens, here we
+            update the selected object to follow suit.
+        """
+        if buff.document.cursor_position_row != my_app.selected_object_idx[0]:
+            my_app.select(buff.document.cursor_position_row)
+
     sidebar_buffer = Buffer(
         name = "sidebarbuffer",
         read_only = True,
@@ -556,19 +552,19 @@ def sql_sidebar(my_app: "sqlApp") -> Window:
     )
 
     class myLexer(Lexer):
-        def __init__(self, token_list = None, *args, **kwargs):
+        def __init__(self, *args, **kwargs):
             super().__init__(*args, **kwargs)
-            self.token_list = token_list
+            self._obj_list = []
+
+        def add_objects(self, objects: List):
+            self._obj_list = objects
 
         def lex_document(self, document: Document) -> Callable[[int], StyleAndTextTuples]:
             def get_line(lineno: int) -> StyleAndTextTuples:
                 # TODO: raise out-of-range exception
-                return self.token_list[lineno]
+                return tokenize_obj(self._obj_list[lineno])
             return get_line
 
-        def reset_tokens(self, tokens: [StyleAndTextTuples]):
-            self.token_list = tokens
-
 
     sidebar_lexer = myLexer()
 
@@ -577,7 +573,7 @@ def sql_sidebar(my_app: "sqlApp") -> Window:
         def move_cursor_down(self):
             my_app.select_next()
         # Need to figure out what do do here
-        # AFAICT thse are only called for the mouse handler
+        # AFAICT these are only called for the mouse handler
         # when events are otherwise not handled
         def move_cursor_up(self):
             my_app.select_previous()
@@ -592,29 +588,28 @@ def sql_sidebar(my_app: "sqlApp") -> Window:
             return NotImplemented
 
         def create_content(self, width: int, height: Optional[int]) -> UIContent:
+            # Only traverse the obj_list if it has been expanded / collapsed
+            if not my_app.obj_list_changed:
+                self.buffer.cursor_position = my_app.selected_object_idx[1]
+                return super().create_content(width, height)
+
             res = []
-            res_tokens = []
-            count = 0
             obj = my_app.obj_list[0]
-            res.append(obj.name)
-            res_tokens.append(tokenize_obj(obj))
-            found_selected = obj is my_app.selected_object
-            idx = 0
+            res.append(obj)
             while obj.next_object is not my_app.obj_list[0]:
-                res.append(obj.next_object.name)
-                res_tokens.append(tokenize_obj(obj.next_object))
-                if obj is not my_app.selected_object and not found_selected:
-                    count += 1
-                    idx += len(obj.name) + 1 # Newline character
-                else:
-                    found_selected = True
+                res.append(obj.next_object)
                 obj = obj.next_object
 
+            self.lexer.add_objects(res)
             self.buffer.set_document(Document(
-                text = "\n".join(res), cursor_position = idx), True)
-            self.lexer.reset_tokens(res_tokens)
+                text = "\n".join([a.name for a in res]), cursor_position = my_app.selected_object_idx[1]), True)
+            # Reset obj_list_changed flag, now that we have had a chance to
+            # regenerate the sidebar document content
+            my_app.obj_list_changed = False
             return super().create_content(width, height)
 
+
+
     sidebar_control = myControl(
             buffer = sidebar_buffer,
             lexer = sidebar_lexer,
